@@ -14,6 +14,8 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2007 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2019 Volker Krause <vkrause@kde.org>
+// Copyright (C) 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -23,17 +25,19 @@
 #ifndef PDFDOCENCODING_H
 #define PDFDOCENCODING_H
 
+#include <string>
+
 #include "CharTypes.h"
 
 class GooString;
 
 #if defined(MIKTEX)
 #include <miktex/poppler/export.h>
-extern MIKTEX_POPPLER_EXPORT Unicode pdfDocEncoding[256]; 
+extern MIKTEX_POPPLER_EXPORT const Unicode pdfDocEncoding[256];
 #else
-extern Unicode pdfDocEncoding[256];
+extern const Unicode pdfDocEncoding[256];
 #endif
 
-char* pdfDocEncodingToUTF16 (GooString* orig, int* length);
+char *pdfDocEncodingToUTF16(const std::string &orig, int *length);
 
 #endif
