@@ -1,6 +1,6 @@
 /* File.cpp: file operations
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -30,6 +30,7 @@
 #include <miktex/Core/File>
 #include <miktex/Core/FileStream>
 #include <miktex/Core/Paths>
+#include <miktex/Util/PathName>
 
 #include "internal.h"
 
@@ -38,6 +39,7 @@
 using namespace std;
 
 using namespace MiKTeX::Core;
+using namespace MiKTeX::Util;
 
 bool File::Exists(const PathName& path)
 {
