@@ -1,6 +1,6 @@
 /* miktex/Core/Session.h:                               -*- C++ -*-
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -38,18 +38,20 @@
 #include <string>
 #include <vector>
 
+#include <miktex/Configuration/ConfigurationProvider>
+#include <miktex/Configuration/HasNamedValues>
+#include <miktex/Configuration/TriState>
+#include <miktex/Core/Text>
+#include <miktex/Util/OptionSet>
+#include <miktex/Util/PathName>
 #include <miktex/Trace/TraceCallback>
 #include <miktex/Util/DateUtil>
 
 #include "Exceptions.h"
 #include "File.h"
 #include "FileType.h"
-#include "HasNamedValues.h"
-#include "OptionSet.h"
-#include "PathName.h"
 #include "Process.h"
 #include "RootDirectoryInfo.h"
-#include "TriState.h"
 #include "VersionNumber.h"
 
 /// @namespace MiKTeX::Core
@@ -109,15 +111,15 @@ struct StartupConfig
 {
   /// User configuration root directory.
 public:
-  PathName userConfigRoot;
+  MiKTeX::Util::PathName userConfigRoot;
 
   /// User data root directory.
 public:
-  PathName userDataRoot;
+  MiKTeX::Util::PathName userDataRoot;
 
   /// User installation root directory.
 public:
-  PathName userInstallRoot;
+  MiKTeX::Util::PathName userInstallRoot;
 
   /// Additional TEXMF roots defined by the user.
 public:
@@ -129,15 +131,15 @@ public:
 
   /// System-wide configuration root directory.
 public:
-  PathName commonConfigRoot;
+  MiKTeX::Util::PathName commonConfigRoot;
 
   /// System-wide data root directory.
 public:
-  PathName commonDataRoot;
+  MiKTeX::Util::PathName commonDataRoot;
 
   /// System-wide installation root directory.
 public:
-  PathName commonInstallRoot;
+  MiKTeX::Util::PathName commonInstallRoot;
 
   /// Additional TEXMF root defined by the admin.
 public:
@@ -208,64 +210,6 @@ public:
   bool isNew = false;
 };
 
-/// Special path names.
-enum class SpecialPath
-{
-  /// User configuration root directory.
-  UserConfigRoot,
-
-  /// User data root directory.
-  UserDataRoot,
-
-  /// User installation root directory.
-  UserInstallRoot,
-
-  /// System-wide configuration root directory.
-  CommonConfigRoot,
-
-  /// System-wide data root directory.
-  CommonDataRoot,
-
-  /// System-wide installation root directory.
-  CommonInstallRoot,
-
-  /// The portable root directory.
-  PortableRoot,
-
-  /// The portable mount directory.
-  PortableMount,
-
-  /// The root directory of the MiKTeX installation.
-  DistRoot,
-
-#if defined(MIKTEX_MACOS_BUNDLE)
-  /// The directory which contains the main application file.
-  /// Example: `/Applications/MiKTeX\ Console.app/Contents/Macos`
-  MacOsDirectory,
-#endif
-
-  /// The effective configuration root directory.
-  ConfigRoot,
-
-  /// The effective data root directory.
-  DataRoot,
-
-  /// The effective installation root directory.
-  InstallRoot,
-
-  /// The effective directory for MiKTeX binaries.
-  BinDirectory,
-
-  /// The effective directory for internal MiKTeX binaries.
-  InternalBinDirectory,
-
-  /// The effective directory in which to create symbolic links to the MiKTeX executables.
-  LinkTargetDirectory,
-
-  /// The effective directory for log files.
-  LogDirectory,
-};
-
 /// Paper size information.
 class PaperSizeInfo
 {
@@ -386,291 +330,6 @@ struct LanguageInfo
   bool custom = false;
 };
 
-/// MiKTeX configuration value.
-class ConfigValue
-{
-public:
-  ConfigValue()
-  {
-  };
-
-public:
-  ConfigValue(const ConfigValue& other)
-  {
-    switch (other.type)
-    {
-    case Type::String:
-      new (&this->s) std::string(other.s);
-      break;
-    case Type::Int:
-      this->i = other.i;
-      break;
-    case Type::Bool:
-      this->b = other.b;
-      break;
-    case Type::Tri:
-      this->t = other.t;
-      break;
-    case Type::Char:
-      this->c = other.c;
-      break;
-    case Type::StringArray:
-      new (&this->sa) std::vector<std::string>(other.sa);
-      break;
-    case Type::None:
-      break;
-    }
-    this->type = other.type;
-    this->section = other.section;
-    this->description = other.description;
-  }
-
-public:
-  ConfigValue& operator=(const ConfigValue& other) = delete;
-
-public:
-  ConfigValue(ConfigValue&& other) noexcept
-  {
-    switch (other.type)
-    {
-    case Type::String:
-      new (&this->s) std::string(other.s);
-      this->s = std::move(other.s);
-      break;
-    case Type::Int:
-      this->i = other.i;
-      break;
-    case Type::Bool:
-      this->b = other.b;
-      break;
-    case Type::Tri:
-      this->t = other.t;
-      break;
-    case Type::Char:
-      this->c = other.c;
-      break;
-    case Type::StringArray:
-      new (&this->sa) std::vector<std::string>(other.sa);
-      this->sa = std::move(other.sa);
-      break;
-    case Type::None:
-      break;
-    }
-    this->type = other.type;
-    this->section = other.section;
-  }
-
-public:
-  ConfigValue& operator=(ConfigValue&& other) noexcept
-  {
-    if (this->type == Type::String && other.type != Type::String)
-    {
-      this->s.~basic_string();
-    }
-    else if (this->type == Type::StringArray && other.type != Type::StringArray)
-    {
-      this->sa.~vector();
-    }
-    switch (other.type)
-    {
-    case Type::String:
-      if (this->type != Type::String)
-      {
-        new (&this->s) std::string(other.s);
-      }
-      this->s = std::move(other.s);
-      break;
-    case Type::Int:
-      this->i = other.i;
-      break;
-    case Type::Bool:
-      this->b = other.b;
-      break;
-    case Type::Tri:
-      this->t = other.t;
-      break;
-    case Type::Char:
-      this->c = other.c;
-      break;
-    case Type::StringArray:
-      if (this->type != Type::StringArray)
-      {
-        new (&this->sa) std::vector<std::string>(other.sa);
-      }
-      this->sa = std::move(other.sa);
-      break;
-    case Type::None:
-      break;
-    }
-    this->type = other.type;
-    this->section = other.section;
-    return *this;
-  }
-
-public:
-  virtual ~ConfigValue() noexcept
-  {
-    if (type == Type::String)
-    {
-      this->s.~basic_string();
-    }
-    else if (type == Type::StringArray)
-    {
-      this->sa.~vector();
-    }
-    type = Type::None;
-  }
-
-public:
-  explicit ConfigValue(const std::string& s)
-  {
-    new(&this->s) std::string(s);
-    type = Type::String;
-  }
-
-public:
-  explicit ConfigValue(const char* lpsz)
-  {
-    new(&this->s) std::string(lpsz == nullptr ? "" : lpsz);
-    type = Type::String;
-  }
-
-public:
-  explicit ConfigValue(int i)
-  {
-    this->i = i;
-    type = Type::Int;
-  }
-
-public:
-  explicit ConfigValue(bool b)
-  {
-    this->b = b;
-    type = Type::Bool;
-  }
-
-public:
-  explicit ConfigValue(TriState t)
-  {
-    this->t = t;
-    type = Type::Tri;
-  }
-
-public:
-  explicit ConfigValue(char c)
-  {
-    this->c = c;
-    type = Type::Char;
-  }
-
-public:
-  explicit ConfigValue(const std::vector<std::string>& sa)
-  {
-    new(&this->sa) std::vector<std::string>(sa);
-    type = Type::StringArray;
-  }
-
-  /// Gets the configuration value as a string.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(std::string) GetString() const;
-
-  /// Gets the configuration value as an integer.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(int) GetInt() const;
-
-  /// Gets the configuration value as a boolean.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(bool) GetBool() const;
-
-  /// Gets the configuration value as a tri-state.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(TriState) GetTriState() const;
-
-  /// Gets the configuration value as a character.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(char) GetChar() const;
-
-  /// Gets the configuration value as a time_t.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(std::time_t) GetTimeT() const;
-
-  /// Gets the configuration value as a string list.
-  /// @return Returns the configuration value.
-public:
-  MIKTEXCORETHISAPI(std::vector<std::string>) GetStringArray() const;
-
-  /// Value type.
-public:
-  enum class Type
-  {
-    None,
-    String,
-    Int,
-    Bool,
-    Tri,
-    Char,
-    StringArray
-  };
-
-private:
-  Type type = Type::None;
-
-  /// Gets the value type.
-  /// @return Returns the value type
-public:
-  Type GetType() const
-  {
-    return type;
-  }
-
-  /// Tests whether the configuration value is defined.
-  /// @return Returns `true`, if the configuration value is defined.
-public:
-  bool HasValue() const
-  {
-    return type != Type::None;
-  }
-
-private:
-  std::string section;
-
-  /// Gets the section of the configuration value.
-  /// @return Returns the section name.
-public:
-  std::string GetSection() const
-  {
-    return section;
-  }
-
-private:
-  std::string description;
-
-  /// Gets the description of the configuration value.
-  /// @return Returns the description.
-public:
-  std::string GetDescription() const
-  {
-    return description;
-  }
-
-private:
-  union
-  {
-    std::string s;
-    int i;
-    bool b;
-    TriState t;
-    char c;
-    std::vector<std::string> sa;
-  };
-};
-
 /// Expansion options.
 enum class ExpandOption
 {
@@ -682,7 +341,7 @@ enum class ExpandOption
   PathPatterns
 };
 
-typedef OptionSet<ExpandOption> ExpandOptionSet;
+typedef MiKTeX::Util::OptionSet<ExpandOption> ExpandOptionSet;
 
 /// Root registration options.
 enum class RegisterRootDirectoriesOption
@@ -697,7 +356,7 @@ enum class RegisterRootDirectoriesOption
   Review
 };
 
-typedef OptionSet<RegisterRootDirectoriesOption> RegisterRootDirectoriesOptionSet;
+typedef MiKTeX::Util::OptionSet<RegisterRootDirectoriesOption> RegisterRootDirectoriesOptionSet;
 
 /// Shell command mode.
 enum class ShellCommandMode
@@ -721,18 +380,19 @@ class MIKTEXNOVTABLE IFindFileCallback
   /// @param[out] installRoot Installation root directory.
   /// @return Returns `true`, if the package has been installed.
 public:
-  virtual bool MIKTEXTHISCALL InstallPackage(const std::string& packageId, const PathName& trigger, PathName& installRoot) = 0;
+  virtual bool MIKTEXTHISCALL InstallPackage(const std::string& packageId, const MiKTeX::Util::PathName& trigger, MiKTeX::Util::PathName& installRoot) = 0;
 
   /// Tries to create a file.
   /// @param fileName Name of the file to create.
   /// @param fileType Type of the file.
   /// @return Returns `true`, if the file has been created.
 public:
-  virtual bool MIKTEXTHISCALL TryCreateFile(const PathName& fileName, FileType fileType) = 0;
+  virtual bool MIKTEXTHISCALL TryCreateFile(const MiKTeX::Util::PathName& fileName, FileType fileType) = 0;
 };
 
 /// The MiKTeX session interface.
-class MIKTEXNOVTABLE Session
+class MIKTEXNOVTABLE Session :
+  public MiKTeX::Configuration::ConfigurationProvider
 {
 public:
   /// Find file options.
@@ -749,7 +409,7 @@ public:
   };
 
 public:
-  typedef OptionSet<FindFileOption> FindFileOptionSet;
+  typedef MiKTeX::Util::OptionSet<FindFileOption> FindFileOptionSet;
 
 public:
   /// Initialization options.
@@ -759,16 +419,12 @@ public:
     SettingUp,
     /// Don't fix `PATH`.
     NoFixPath,
-#if defined(MIKTEX_WINDOWS)
-    /// Initialize the COM library.
-    InitializeCOM,
-#endif
     /// Start in administrator mode.
     AdminMode,
   };
 
   /// Init flags enum.
-  typedef OptionSet<InitOption> InitOptionSet;
+  typedef MiKTeX::Util::OptionSet<InitOption> InitOptionSet;
 
   /// Extended initializion options.
 public:
@@ -980,13 +636,7 @@ public:
   /// @param path The file system path to the directory.
   /// @param atEnd Indicates wheter the directory shall be added at the end of the list.
 public:
-  virtual void MIKTEXTHISCALL AddInputDirectory(const PathName& path, bool atEnd) = 0;
-
-  /// Gets a special file system path.
-  /// @param specialPath Identifies the special path.
-  /// @return Returns a path name.
-public:
-  virtual PathName MIKTEXTHISCALL GetSpecialPath(SpecialPath specialPath) = 0;
+  virtual void MIKTEXTHISCALL AddInputDirectory(const MiKTeX::Util::PathName& path, bool atEnd) = 0;
 
   /// Gets registered root directories.
   /// @return Returns the list of registered root directories.
@@ -1002,7 +652,7 @@ public:
   /// @param r Identifies the root directory.
   /// @return Returns the file system path to the root directory.
 public:
-  virtual PathName MIKTEXTHISCALL GetRootDirectoryPath(unsigned r) = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetRootDirectoryPath(unsigned r) = 0;
 
   /// Tests whether a root directory is registered system-wide.
   /// @param r Identifies the root directory.
@@ -1019,37 +669,37 @@ public:
   /// Gets the virtual path to the MPM root directory.
   /// @return Returns the virtual path to the MPM root directory.
 public:
-  virtual PathName MIKTEXTHISCALL GetMpmRootPath() = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetMpmRootPath() = 0;
 
   /// Gets a path to the MPM file name database.
   /// @return Returns the path to the MPM file name database.
 public:
-  virtual PathName MIKTEXTHISCALL GetMpmDatabasePathName() = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetMpmDatabasePathName() = 0;
 
   /// Tries to get a root directory index from a file system path.
   /// @param path The file system path.
   /// @return Returns a root directory index (or `INVALID_ROOT_INDEX`).
 public:
-  virtual unsigned MIKTEXTHISCALL TryDeriveTEXMFRoot(const PathName& path) = 0;
+  virtual unsigned MIKTEXTHISCALL TryDeriveTEXMFRoot(const MiKTeX::Util::PathName& path) = 0;
 
   /// Get a root directory index from a file system path.
   /// @param path The file system path.
   /// @return Returns a root directory index.
 public:
-  virtual unsigned MIKTEXTHISCALL DeriveTEXMFRoot(const PathName& path) = 0;
+  virtual unsigned MIKTEXTHISCALL DeriveTEXMFRoot(const MiKTeX::Util::PathName& path) = 0;
 
   /// Finds a file name database.
   /// @param r Identifies the root directory.
   /// @param[out] path The file system path to the databsae.
   /// @return Returns `true`, if the database was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFilenameDatabase(unsigned r, PathName& path) = 0;
+  virtual bool MIKTEXTHISCALL FindFilenameDatabase(unsigned r, MiKTeX::Util::PathName& path) = 0;
 
   /// Gets the file system path to a file name database file.
   /// @param r Identifies the root directory.
   /// @return Returns the file system path to the databsae.
 public:
-  virtual PathName MIKTEXTHISCALL GetFilenameDatabasePathName(unsigned r) = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetFilenameDatabasePathName(unsigned r) = 0;
 
   /// Unloads the file name database.
   /// @return Returns `true`, if the file name database could be unloaded.
@@ -1070,7 +720,7 @@ public:
   /// @param[out] relative The relative path from the root directory.
   /// @return Returns the root directory index.
 public:
-  virtual unsigned MIKTEXTHISCALL SplitTEXMFPath(const PathName& path, PathName& root, PathName& relative) = 0;
+  virtual unsigned MIKTEXTHISCALL SplitTEXMFPath(const MiKTeX::Util::PathName& path, MiKTeX::Util::PathName& root, MiKTeX::Util::PathName& relative) = 0;
 
   /// Registers TEXMF root directories.
   /// @param startupConfig Specifies the root directories to register.
@@ -1082,13 +732,13 @@ public:
   /// @param path The file system path to the root directory.
   /// @param other Indicates wheter the root directory is from another TeX system.
 public:
-virtual void MIKTEXTHISCALL RegisterRootDirectory(const PathName& path, bool other) = 0;
+virtual void MIKTEXTHISCALL RegisterRootDirectory(const MiKTeX::Util::PathName& path, bool other) = 0;
 
   /// Unregisters an additional root directory.
   /// @param path The file system path to the root directory.
   /// @param other Indicates wheter the root directory is from another TeX system.
 public:
-virtual void MIKTEXTHISCALL UnregisterRootDirectory(const PathName& path, bool other) = 0;
+virtual void MIKTEXTHISCALL UnregisterRootDirectory(const MiKTeX::Util::PathName& path, bool other) = 0;
 
   /// Moves a root directory up in the list.
   /// @param r Identifies the root directory.
@@ -1124,67 +774,6 @@ public:
 public:
   virtual bool MIKTEXTHISCALL DetermineMETAFONTMode(unsigned dpi, MIKTEXMFMODE& Mode) = 0;
 
-  /// Tries to get a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @param callback The pointer to an object which implements the `HasNamedValue` interface.
-  /// @param[out] value The configuration value as a string.
-  /// @return Returns `true`, if the value was found.
-public:
-  virtual bool MIKTEXTHISCALL TryGetConfigValue(const std::string& sectionName, const std::string& valueName, HasNamedValues* callback, std::string& value) = 0;
-
-  /// Tries to get a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @param[out] value The configuration value as a string.
-  /// @return Returns `true`, if the value was found.
-public:
-  virtual bool MIKTEXTHISCALL TryGetConfigValue(const std::string& sectionName, const std::string& valueName, std::string& value) = 0;
-
-  /// Gets a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @param defaultValue Value to be returned if the requested value was not found.
-  /// @param callback The pointer to an object which implements the `HasNamedValue` interface.
-  /// @return Returns the configuration value.
-  /// @see SetConfigValue
-public:
-  virtual ConfigValue MIKTEXTHISCALL GetConfigValue(const std::string& sectionName, const std::string& valueName, const ConfigValue& defaultValue, HasNamedValues* callback) = 0;
-
-  /// Gets a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @param defaultValue Value to be returned if the requested value was not found.
-  /// @return Returns the configuration value.
-  /// @see SetConfigValue
-public:
-  virtual ConfigValue MIKTEXTHISCALL GetConfigValue(const std::string& sectionName, const std::string& valueName, const ConfigValue& defaultValue) = 0;
-
-  /// Gets a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @param callback The pointer to an object which implements the `HasNamedValue` interface.
-  /// @return Returns the configuration value.
-  /// @see SetConfigValue
-public:
-  virtual ConfigValue MIKTEXTHISCALL GetConfigValue(const std::string& sectionName, const std::string& valueName, HasNamedValues* callback) = 0;
-
-  /// Gets a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @return Returns the configuration value.
-  /// @see SetConfigValue
-public:
-  virtual ConfigValue MIKTEXTHISCALL GetConfigValue(const std::string& sectionName, const std::string& valueName) = 0;
-
-  /// Sets a configuration value.
-  /// @param sectionName Identifies the configuration section.
-  /// @param valueName Identifies the value within the section.
-  /// @param The configuration value.
-  /// @see GetConfigValue
-public:
-  virtual void MIKTEXTHISCALL SetConfigValue(const std::string& sectionName, const std::string& valueName, const ConfigValue& value) = 0;
-
   /// Gets the name of the running engine (e.g., `pdftex`).
   /// @return Returns the engine name.
 public:
@@ -1197,7 +786,7 @@ public:
   /// @param isTextFile Specifies if the file should be opened in text mode.
   /// @return Returns the pointer to a `FILE` object.
 public:
-  virtual FILE* MIKTEXTHISCALL OpenFile(const PathName& path, FileMode mode, FileAccess access, bool isTextFile) = 0;
+  virtual FILE* MIKTEXTHISCALL OpenFile(const MiKTeX::Util::PathName& path, FileMode mode, FileAccess access, bool isTextFile) = 0;
 
   /// Tries to open a file.
   /// @param path The file system path to the file.
@@ -1207,7 +796,7 @@ public:
   /// @return Returns the pointer to a `FILE` object. Returns `nullptr`, if
   /// the file could not be opened.
 public:
-  virtual FILE* MIKTEXTHISCALL TryOpenFile(const PathName& path, FileMode mode, FileAccess access, bool isTextFile) = 0;
+  virtual FILE* MIKTEXTHISCALL TryOpenFile(const MiKTeX::Util::PathName& path, FileMode mode, FileAccess access, bool isTextFile) = 0;
 
   /// Tries to get information about an open file.
   /// @param file The pointer to the `FILE` object.
@@ -1238,7 +827,7 @@ public:
   /// @param fileName Name of the file to be checked.
   /// @return Returns true, if the file has been opened.
 public:
-  virtual bool MIKTEXTHISCALL IsFileAlreadyOpen(const PathName& fileName) = 0;
+  virtual bool MIKTEXTHISCALL IsFileAlreadyOpen(const MiKTeX::Util::PathName& fileName) = 0;
 #endif
 
   /// Schedules the execution of a shell command when the application terminates.
@@ -1250,7 +839,7 @@ public:
   /// Schedules the removal of a file when the application terminates.
   /// @param path The file system path to the file.
 public:
-  virtual void MIKTEXTHISCALL ScheduleFileRemoval(const PathName& path) = 0;
+  virtual void MIKTEXTHISCALL ScheduleFileRemoval(const MiKTeX::Util::PathName& path) = 0;
 #endif
 
   /// Starts recording file names
@@ -1267,13 +856,13 @@ public:
   /// Sets the file name recorder log file.
   /// @param path The file system path to the log file.
 public:
-  virtual void MIKTEXTHISCALL SetRecorderPath(const PathName& path) = 0;
+  virtual void MIKTEXTHISCALL SetRecorderPath(const MiKTeX::Util::PathName& path) = 0;
 
   /// Adds a file name record to the log file.
   /// @param The file system file to the file.
   /// @param How the file is accessed.
 public:
-  virtual void MIKTEXTHISCALL RecordFileInfo(const PathName& path, FileAccess access) = 0;
+  virtual void MIKTEXTHISCALL RecordFileInfo(const MiKTeX::Util::PathName& path, FileAccess access) = 0;
 
   /// Gets the recorded file names.
   /// @return Returns the recorded file names.
@@ -1284,7 +873,7 @@ public:
   /// @param fileName The file name.
   /// @return Returns the derived file type.
 public:
-  virtual FileType MIKTEXTHISCALL DeriveFileType(const PathName& fileName) = 0;
+  virtual FileType MIKTEXTHISCALL DeriveFileType(const MiKTeX::Util::PathName& fileName) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1293,7 +882,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, FindFileOptionSet options, std::vector<PathName>& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, FindFileOptionSet options, std::vector<MiKTeX::Util::PathName>& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1301,7 +890,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, std::vector<PathName>& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, std::vector<MiKTeX::Util::PathName>& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1310,7 +899,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, FindFileOptionSet options, PathName& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, FindFileOptionSet options, MiKTeX::Util::PathName& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1318,7 +907,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, PathName& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, const std::string& pathList, MiKTeX::Util::PathName& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1327,7 +916,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, FindFileOptionSet options, std::vector<PathName>& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, FindFileOptionSet options, std::vector<MiKTeX::Util::PathName>& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1335,7 +924,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, std::vector<PathName>& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, std::vector<MiKTeX::Util::PathName>& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1344,7 +933,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, FindFileOptionSet options, PathName& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, FindFileOptionSet options, MiKTeX::Util::PathName& result) = 0;
 
   /// Searches a file.
   /// @param fileName The name of the file to search.
@@ -1352,7 +941,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, PathName& result) = 0;
+  virtual bool MIKTEXTHISCALL FindFile(const std::string& fileName, FileType fileType, MiKTeX::Util::PathName& result) = 0;
 
   /// Searches a PK font file.
   /// @param fontName The name of the font to search.
@@ -1361,7 +950,7 @@ public:
   /// @param[out] result The result of the search.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindPkFile(const std::string& fontName, const std::string& mfMode, int dpi, PathName& result) = 0;
+  virtual bool MIKTEXTHISCALL FindPkFile(const std::string& fontName, const std::string& mfMode, int dpi, MiKTeX::Util::PathName& result) = 0;
 
   /// Searches a TFM file.
   /// @param fontName The name of the font to search.
@@ -1369,7 +958,7 @@ public:
   /// @param create Indicates, if the TFM file should be created if it was not found.
   /// @return Return `true`, if the file was found.
 public:
-  virtual bool MIKTEXTHISCALL FindTfmFile(const std::string& fontName, PathName& result, bool create) = 0;
+  virtual bool MIKTEXTHISCALL FindTfmFile(const std::string& fontName, MiKTeX::Util::PathName& result, bool create) = 0;
 
   /// Sets the callback interface for file search.
   /// @param callback The pointer to an object which implements the interface.
@@ -1384,7 +973,7 @@ public:
   /// @param[out] fontName The font name.
   /// @param[oit] pointSize The size of the font.
 public:
-  virtual void MIKTEXTHISCALL SplitFontPath(const PathName& fontPath, std::string* fontType, std::string* supplier, std::string* typeface, std::string* fontName, std::string* pointSize) = 0;
+  virtual void MIKTEXTHISCALL SplitFontPath(const MiKTeX::Util::PathName& fontPath, std::string* fontType, std::string* supplier, std::string* typeface, std::string* fontName, std::string* pointSize) = 0;
 
   /// Searches a font file.
   /// @param fontName The name of the font to search.
@@ -1399,7 +988,7 @@ public:
   /// @param[out] versionNumber The Ghostscript version number
   /// @return Returns the file system path to the Ghostscript program file.
 public:
-  virtual PathName MIKTEXTHISCALL GetGhostscript(unsigned long* versionNumber) = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetGhostscript(unsigned long* versionNumber) = 0;
 
   /// Gets the seach path for a file type.
   /// @param fileType The file type.
@@ -1413,7 +1002,7 @@ public:
   /// @param callback The pointer to an object implementing the `IRunProcessCallback` callback.
   /// @return Returns `true`, if the conversion was successful.
 public:
-  virtual bool MIKTEXTHISCALL ConvertToBitmapFile(const PathName& sourceFileName, PathName& destFileName, IRunProcessCallback* callback) = 0;
+  virtual bool MIKTEXTHISCALL ConvertToBitmapFile(const MiKTeX::Util::PathName& sourceFileName, MiKTeX::Util::PathName& destFileName, IRunProcessCallback* callback) = 0;
 
   /// Enables or disables automatic font file creation.
   /// @param enable Indicates whether automatic font file creation is enabled.
@@ -1435,7 +1024,7 @@ public:
   /// @param enableInstaller Indicates whether automatic package installation is allowed.
   /// @return Returns the `makepk` command-line arguments.
 public:
-  virtual std::vector<std::string> MIKTEXTHISCALL MakeMakePkCommandLine(const std::string& fontName, int dpi, int baseDpi, const std::string& mfMode, PathName& fileName, TriState enableInstaller) = 0;
+  virtual std::vector<std::string> MIKTEXTHISCALL MakeMakePkCommandLine(const std::string& fontName, int dpi, int baseDpi, const std::string& mfMode, MiKTeX::Util::PathName& fileName, MiKTeX::Configuration::TriState enableInstaller) = 0;
 
 #if defined(MIKTEX_WINDOWS)
   /// Executes a Windows batch script.
@@ -1541,19 +1130,19 @@ public:
   /// @param canonicalized Indicates wheter symbolic links should be followed.
   /// @return Returns the file system path to the running program file.
 public:
-  virtual PathName MIKTEXTHISCALL GetMyProgramFile(bool canonicalized) = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetMyProgramFile(bool canonicalized) = 0;
 
   /// Gets the file system path to the directory of the running program file.
   /// @param canonicalized Indicates wheter symbolic links should be followed.
   /// @return Returns the file system path to the directory of running program file.
 public:
-  virtual PathName MIKTEXTHISCALL GetMyLocation(bool canonicalized) = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetMyLocation(bool canonicalized) = 0;
 
   /// Gets the file system path to the prefix directory of the running program file.
   /// @param canonicalized Indicates wheter symbolic links should be followed.
   /// @return Returns the file system path to the prefix directory.
 public:
-  virtual PathName MIKTEXTHISCALL GetMyPrefix(bool canonicalized) = 0;
+  virtual MiKTeX::Util::PathName MIKTEXTHISCALL GetMyPrefix(bool canonicalized) = 0;
 
   /// Tests whether the program runs with elevated privileges.
   /// @return Returns `true`, if the program is running with elevated privileges.
@@ -1603,7 +1192,7 @@ public:
   /// @param The file system path to the file.
   /// @return Returns `true`, if the file has been created.
 public:
-  virtual bool MIKTEXTHISCALL TryCreateFromTemplate(const PathName& path) = 0;
+  virtual bool MIKTEXTHISCALL TryCreateFromTemplate(const MiKTeX::Util::PathName& path) = 0;
 
   /// Tests whether the current user is an administrator.
   /// @return Returns `true`, if the current user is an administrator.
@@ -1615,13 +1204,13 @@ public:
   /// @param pathOut The file system path to the destination file.
   /// @param callback The pointer to an object which implements the `HasNamedValue` interface.
 public:
-  virtual void MIKTEXTHISCALL ConfigureFile(const PathName& pathIn, const PathName& pathOut, HasNamedValues* callback = nullptr) = 0;
+  virtual void MIKTEXTHISCALL ConfigureFile(const MiKTeX::Util::PathName& pathIn, const MiKTeX::Util::PathName& pathOut, MiKTeX::Configuration::HasNamedValues* callback = nullptr) = 0;
 
   /// Configures a file.
   /// @param pathRel The relative file system path to the destination file.
   /// @param callback The pointer to an object which implements the `HasNamedValue` interface.
 public:
-  virtual void MIKTEXTHISCALL ConfigureFile(const PathName& pathRel, HasNamedValues* callback = nullptr) = 0;
+  virtual void MIKTEXTHISCALL ConfigureFile(const MiKTeX::Util::PathName& pathRel, MiKTeX::Configuration::HasNamedValues* callback = nullptr) = 0;
 
   /// Sets the descriptive name of the running program.
   /// @param name The descriptive name.
@@ -1650,7 +1239,7 @@ public:
   /// @param callback The pointer to an object which implements the `HasNamedValues` interface.
   /// @return Returns the expanded string.
 public:
-  virtual std::string Expand(const std::string& toBeExpanded, HasNamedValues* callback) = 0;
+  virtual std::string Expand(const std::string& toBeExpanded, MiKTeX::Configuration::HasNamedValues* callback) = 0;
 
   /// Expands a string.
   /// @param toBeExpanded The string to expand.
@@ -1658,7 +1247,7 @@ public:
   /// @param callback The pointer to an object which implements the `HasNamedValues` interface.
   /// @return Returns the expanded string.
 public:
-  virtual std::string Expand(const std::string& toBeExpanded, ExpandOptionSet options, HasNamedValues* callback) = 0;
+  virtual std::string Expand(const std::string& toBeExpanded, ExpandOptionSet options, MiKTeX::Configuration::HasNamedValues* callback) = 0;
 
   /// Updates language data.
   /// @param languageInfo The language data.
