@@ -1,6 +1,6 @@
 /* SetupService.cpp:
 
-   Copyright (C) 2013-2020 Christian Schenk
+   Copyright (C) 2013-2021 Christian Schenk
 
    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -32,8 +32,10 @@ using namespace std::string_literals;
 
 using namespace nlohmann;
 
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Extractor;
+using namespace MiKTeX::Locale;
 using namespace MiKTeX::Packages;
 using namespace MiKTeX::Setup;
 using namespace MiKTeX::Trace;
@@ -109,14 +111,16 @@ string IssueSeverityString(IssueSeverity severity)
 {
   switch (severity)
   {
-  case IssueSeverity::Critical: return T_("critical issue");
-  case IssueSeverity::Major: return T_("major issue");
-  case IssueSeverity::Minor: return T_("minor issue");
-  case IssueSeverity::Trivial: return T_("trivial issue");
+  case IssueSeverity::Critical: return "critical issue";
+  case IssueSeverity::Major: return "major issue";
+  case IssueSeverity::Minor: return "minor issue";
+  case IssueSeverity::Trivial: return "trivial issue";
   default: MIKTEX_UNEXPECTED();
   }
 }
 
+SetupResources SetupServiceImpl::resources;
+
 END_INTERNAL_NAMESPACE;
 
 namespace MiKTeX {
@@ -163,9 +167,10 @@ SetupServiceImpl::SetupServiceImpl()
 {
   traceStream = TraceStream::Open("setup");
   packageManager = PackageManager::Create();
-  shared_ptr<Session> session = Session::Get();
+  session = Session::Get();
   logFile.SetCallback(this);
   options.IsCommonSetup = session->RunningAsAdministrator();
+  translator = make_unique<Translator>(MIKTEX_COMP_ID, &resources, session);
 }
 
 SetupServiceImpl::~SetupServiceImpl()
@@ -501,13 +506,12 @@ PathName SetupServiceImpl::CloseLog(bool cancel)
 
 void SetupServiceImpl::LogHeader()
 {
-  Log(fmt::format(T_("{0} {1} Report\n\n"), options.Banner, options.Version));
+  Log(fmt::format("{0} {1} Report\n\n", options.Banner, options.Version));
   time_t t = time(nullptr);
   struct tm* pTm = localtime(&t);
-  Log(fmt::format(T_("Date: {0:%A, %B %d, %Y}\n"), *pTm));
-  Log(fmt::format(T_("Time: {0:%H:%M:%S}\n"), *pTm));
-  Log(fmt::format(T_("OS version: {0}\n"), Utils::GetOSVersionString()));
-  shared_ptr<Session> session = Session::Get();
+  Log(fmt::format("Date: {0:%A, %B %d, %Y}\n", *pTm));
+  Log(fmt::format("Time: {0:%H:%M:%S}\n", *pTm));
+  Log(fmt::format("OS version: {0}\n", Utils::GetOSVersionString()));
   Log(fmt::format("SystemAdmin: {}\n", session->RunningAsAdministrator()));
   if (options.Task != SetupTask::Download)
   {
@@ -519,24 +523,24 @@ void SetupServiceImpl::LogHeader()
   {
     MIKTEX_FATAL_WINDOWS_ERROR("GetModuleFileNameW");
   }
-  Log(fmt::format(T_("Setup path: {0}\n"), WU_(szSetupPath)));
+  Log(fmt::format("Setup path: {0}\n", WU_(szSetupPath)));
 #else
   // TODO: log setup path
 #endif
   if (options.Task != SetupTask::Download)
   {
-    Log(fmt::format("UserRoots: {}\n", options.Config.userRoots.empty() ? T_("<none specified>") : options.Config.userRoots));
-    Log(fmt::format("UserData: {}\n", options.Config.userDataRoot.Empty() ? T_("<none specified>") : options.Config.userDataRoot.ToString()));
-    Log(fmt::format("UserConfig: {}\n", options.Config.userConfigRoot.Empty() ? T_("<none specified>") : options.Config.userConfigRoot.ToString()));
-    Log(fmt::format("CommonRoots: {}\n", options.Config.commonRoots.empty() ? T_("<none specified>") : options.Config.commonRoots));
-    Log(fmt::format("CommonData: {}\n", options.Config.commonDataRoot.Empty() ? T_("<none specified>") : options.Config.commonDataRoot.ToString()));
-    Log(fmt::format("CommonConfig: {}\n", options.Config.commonConfigRoot.Empty() ? T_("<none specified>") : options.Config.commonConfigRoot.ToString()));
+    Log(fmt::format("UserRoots: {}\n", options.Config.userRoots.empty() ? "<none specified>" : options.Config.userRoots));
+    Log(fmt::format("UserData: {}\n", options.Config.userDataRoot.Empty() ? "<none specified>" : options.Config.userDataRoot.ToString()));
+    Log(fmt::format("UserConfig: {}\n", options.Config.userConfigRoot.Empty() ? "<none specified>" : options.Config.userConfigRoot.ToString()));
+    Log(fmt::format("CommonRoots: {}\n", options.Config.commonRoots.empty() ? "<none specified>" : options.Config.commonRoots));
+    Log(fmt::format("CommonData: {}\n", options.Config.commonDataRoot.Empty() ? "<none specified>" : options.Config.commonDataRoot.ToString()));
+    Log(fmt::format("CommonConfig: {}\n", options.Config.commonConfigRoot.Empty() ? "<none specified>" : options.Config.commonConfigRoot.ToString()));
     PathName installRoot = GetInstallRoot();
-    Log(fmt::format("Installation: {}\n", installRoot.Empty() ? T_("<none specified>") : installRoot.ToString()));
+    Log(fmt::format("Installation: {}\n", installRoot.Empty() ? "<none specified>" : installRoot.ToString()));
   }
 }
 
-void SetupServiceImpl::Log(const string& s)
+void SetupServiceImpl::LogInternal(TraceLevel level, const string& s)
 {
 #if 0
   lock_guard<mutex> lockGuard(logStreamMutex);
@@ -551,7 +555,7 @@ void SetupServiceImpl::Log(const string& s)
   {
     if (lpsz[0] == '\n' || (lpsz[0] == '\r' && lpsz[1] == '\n'))
     {
-      traceStream->WriteLine("setup", currentLine);
+      traceStream->WriteLine("setup", level, currentLine);
       if (logStream.is_open())
       {
         logStream << currentLine << "\n";
@@ -670,8 +674,6 @@ void SetupServiceImpl::Run()
 
 void SetupServiceImpl::CompleteOptions(bool allowRemoteCalls)
 {
-  shared_ptr<Session> session = Session::Get();
-
   if (options.Task == SetupTask::FinishUpdate || options.Task == SetupTask::CleanUp)
   {
     options.IsCommonSetup = session->IsSharedSetup();
@@ -741,7 +743,7 @@ void SetupServiceImpl::CompleteOptions(bool allowRemoteCalls)
       {
         if (foundPackageLevel < options.PackageLevel)
         {
-          MIKTEX_FATAL_ERROR(T_("no local package directory found"));
+          MIKTEX_FATAL_ERROR("no local package directory found");
         }
       }
       if (options.PackageLevel == PackageLevel::None)
@@ -765,8 +767,6 @@ void SetupServiceImpl::CompleteOptions(bool allowRemoteCalls)
 
 void SetupServiceImpl::Initialize()
 {
-  shared_ptr<Session> session = Session::Get();
-
   if (initialized)
   {
     return;
@@ -813,8 +813,6 @@ void SetupServiceImpl::DoTheDownload()
 {
   ReportLine("starting downloader...");
 
-  shared_ptr<Session> session = Session::Get();
-
   // remember local repository folder
   session->SetConfigValue(MIKTEX_CONFIG_SECTION_MPM, MIKTEX_CONFIG_VALUE_LOCAL_REPOSITORY, ConfigValue(options.LocalPackageRepository.ToString()));
 
@@ -924,7 +922,6 @@ void SetupServiceImpl::DoTheInstallation()
   {
     startupConfig.userInstallRoot = options.Config.userInstallRoot;
   }
-  shared_ptr<Session> session = Session::Get();
   RegisterRootDirectoriesOptionSet regOptions = { RegisterRootDirectoriesOption::Temporary };
 #if defined(MIKTEX_WINDOWS)
   regOptions += RegisterRootDirectoriesOption::NoRegistry;
@@ -944,7 +941,7 @@ void SetupServiceImpl::DoTheInstallation()
     isArchive = true;
     pathDB = options.LocalPackageRepository / PathName(MIKTEX_PACKAGE_MANIFESTS_ARCHIVE_FILE_NAME);
   }
-  ReportLine(T_("Loading package database..."));
+  ReportLine("Loading package database...");
   packageManager->LoadDatabase(pathDB, isArchive);
 
   // create the destination directory
@@ -1055,7 +1052,6 @@ void SetupServiceImpl::DoFinishSetup()
 void SetupServiceImpl::DoFinishUpdate()
 {
   ReportLine("finishing update...");
-  shared_ptr<Session> session = Session::Get();
   RemoveFormatFiles();
 #if defined(MIKTEX_WINDOWS)
   RunMpm({ "--register-components" });
@@ -1078,8 +1074,6 @@ void SetupServiceImpl::DoFinishUpdate()
 
 void SetupServiceImpl::DoCleanUp()
 {
-  shared_ptr<Session> session = Session::Get();
-
 #if defined(MIKTEX_WINDOWS)
   logFile.Load(session->GetSpecialPath(SpecialPath::InstallRoot) / PathName(MIKTEX_PATH_UNINST_LOG));
 #endif
@@ -1165,7 +1159,6 @@ void SetupServiceImpl::DoCleanUp()
     {
       PathName parent;
       vector<PathName> roots = GetRoots();
-      shared_ptr<Session> session = Session::Get();
       session->UnloadFilenameDatabase();
       for (const PathName& root : roots)
       {
@@ -1276,7 +1269,6 @@ void SetupServiceImpl::DoCleanUp()
 vector<PathName> SetupServiceImpl::GetRoots()
 {
   vector<PathName> vec;
-  shared_ptr<Session> session = Session::Get();
   if (!session->IsMiKTeXDirect())
   {
     PathName installRoot = session->GetSpecialPath(SpecialPath::InstallRoot);
@@ -1313,10 +1305,8 @@ vector<PathName> SetupServiceImpl::GetRoots()
 
 void SetupServiceImpl::UnregisterComponents()
 {
-  shared_ptr<Session> session = Session::Get();
   if (session->IsAdminMode())
   {
-    std::shared_ptr<MiKTeX::Packages::PackageManager> packageManager(PackageManager::Create());
     shared_ptr<PackageInstaller> packageInstaller(packageManager->CreateInstaller());
     packageInstaller->RegisterComponents(false);
     packageInstaller->Dispose();
@@ -1495,7 +1485,6 @@ PathName SetupServiceImpl::GetInstallRoot() const
   }
   else if (options.Task == SetupTask::FinishSetup || options.Task == SetupTask::FinishUpdate || options.Task == SetupTask::CleanUp)
   {
-    shared_ptr<Session> session = Session::Get();
     return session->GetSpecialPath(SpecialPath::InstallRoot);
   }
   else
@@ -1508,7 +1497,6 @@ PathName SetupServiceImpl::GetBinDir() const
 {
   if (options.Task == SetupTask::FinishSetup || options.Task == SetupTask::FinishUpdate || options.Task == SetupTask::CleanUp)
   {
-    shared_ptr<Session> session = Session::Get();
     return session->GetSpecialPath(SpecialPath::BinDirectory);
   }
   else
@@ -1519,8 +1507,6 @@ PathName SetupServiceImpl::GetBinDir() const
 
 void SetupServiceImpl::RunIniTeXMF(const vector<string>& args, bool mustSucceed)
 {
-  shared_ptr<Session> session = Session::Get();
-
   // make absolute exe path name
   PathName exePath = GetBinDir() / PathName(MIKTEX_INITEXMF_EXE);
 
@@ -1564,7 +1550,6 @@ void SetupServiceImpl::RunIniTeXMF(const vector<string>& args, bool mustSucceed)
 
 void SetupServiceImpl::RunMpm(const vector<string>& args)
 {
-  shared_ptr<Session> session = Session::Get();
   // make absolute exe path name
   PathName exePath = GetBinDir() / PathName(MIKTEX_MPM_EXE);
 
@@ -1605,7 +1590,7 @@ void SetupServiceImpl::CreateInfoFile()
   default:
     MIKTEX_ASSERT(false);
   }
-  stream.WriteLine(fmt::format(T_("This folder contains the {0} package set."), lpszPackageSet));
+  stream.WriteLine(fmt::format("This folder contains the {0} package set.", lpszPackageSet));
   stream.WriteLine();
   stream.WriteLine();
 #if defined(MIKTEX_WINDOWS)
@@ -1616,11 +1601,11 @@ void SetupServiceImpl::CreateInfoFile()
   }
   PathName setupExe(szSetupPath);
   setupExe.RemoveDirectorySpec();
-  stream.WriteLine(fmt::format(T_("To install MiKTeX, run {0}."), setupExe));
+  stream.WriteLine(fmt::format("To install MiKTeX, run {0}.", setupExe));
   stream.WriteLine();
   stream.WriteLine();
 #endif
-  stream.WriteLine(T_("For more information, visit the MiKTeX project page at\n\nhttps://miktex.org."));
+  stream.WriteLine("For more information, visit the MiKTeX project page at\n\nhttps://miktex.org.");
   stream.WriteLine();
   stream.Close();
   RepositoryInfo repositoryInfo;
@@ -1748,8 +1733,6 @@ wstring& SetupServiceImpl::Expand(const string& source, wstring& dest)
 
 bool SetupServiceImpl::FindFile(const PathName& fileName, PathName& result)
 {
-  shared_ptr<Session> session = Session::Get();
-
   // try my directory
   result = session->GetMyLocation(false) / fileName;
   if (File::Exists(result))
@@ -1770,7 +1753,6 @@ bool SetupServiceImpl::FindFile(const PathName& fileName, PathName& result)
 
 void SetupServiceImpl::RemoveFormatFiles()
 {
-  shared_ptr<Session> session = Session::Get();
   vector<PathName> toBeDeleted;
   PathName pathFmt(session->GetSpecialPath(SpecialPath::DataRoot));
   pathFmt /= MIKTEX_PATH_FMT_DIR;
@@ -1842,13 +1824,11 @@ constexpr time_t ONE_DAY = 86400;
 
 inline string FormatTimestamp(time_t t)
 {
-  return IsValidTimeT(t) ? fmt::format("{:%F %T}", *localtime(&t)) : T_("not yet");
+  return IsValidTimeT(t) ? fmt::format("{:%F %T}", *localtime(&t)) : "not yet";
 }
 
-void SetupService::WriteReport(ostream& s, ReportOptionSet options)
+void SetupServiceImpl::WriteReport(ostream& s, ReportOptionSet options)
 {
-  shared_ptr<Session> session = Session::Get();
-  shared_ptr<PackageManager> packageManager = PackageManager::Create();
   time_t now = time(nullptr);
   if (options[ReportOption::General])
   {
@@ -1864,9 +1844,9 @@ void SetupService::WriteReport(ostream& s, ReportOptionSet options)
       s << "GitInfo: " << Utils::GetGitInfo() << "\n";
     }
     s << "OS: " << Utils::GetOSVersionString() << "\n"
-      << "SharedSetup: " << (session->IsSharedSetup() ? T_("yes") : T_("no")) << "\n"
+      << "SharedSetup: " << (session->IsSharedSetup() ? "yes" : "no") << "\n"
       << "LinkTargetDirectory: " << session->GetSpecialPath(SpecialPath::LinkTargetDirectory) << "\n"
-      << "PathOkay: " << (p.first ? T_("yes") : T_("no")) << "\n";
+      << "PathOkay: " << (p.first ? "yes" : "no") << "\n";
     if (session->IsSharedSetup())
     {
       InstallationSummary commonInstallation = packageManager->GetInstallationSummary(false);
@@ -1888,9 +1868,9 @@ void SetupService::WriteReport(ostream& s, ReportOptionSet options)
   }
   if (options[ReportOption::CurrentUser])
   {
-    s << "SystemAdmin: " << (session->IsUserAnAdministrator() ? T_("yes") : T_("no")) << "\n"
-      << "RootPrivileges: " << (session->RunningAsAdministrator() ? T_("yes") : T_("no")) << "\n"
-      << "AdminMode: " << (session->IsAdminMode() ? T_("yes") : T_("no")) << "\n";
+    s << "SystemAdmin: " << (session->IsUserAnAdministrator() ? "yes" : "no") << "\n"
+      << "RootPrivileges: " << (session->RunningAsAdministrator() ? "yes" : "no") << "\n"
+      << "AdminMode: " << (session->IsAdminMode() ? "yes" : "no") << "\n";
   }
   if (options[ReportOption::RootDirectories])
   {
@@ -1938,16 +1918,14 @@ void SetupService::WriteReport(ostream& s, ReportOptionSet options)
   }
 }
 
-void SetupService::WriteReport(ostream& s)
+void SetupServiceImpl::WriteReport(ostream& s)
 {
   WriteReport(s, { ReportOption::General, ReportOption::RootDirectories, ReportOption::CurrentUser });
 }
 
-vector<Issue> SetupService::FindIssues(bool checkPath, bool checkPackageIntegrity)
+vector<Issue> SetupServiceImpl::FindIssues(bool checkPath, bool checkPackageIntegrity)
 {
   vector<Issue> result;
-  shared_ptr<Session> session = Session::Get();
-  shared_ptr<PackageManager> packageManager = PackageManager::Create();
   time_t now = time(nullptr);
   if (checkPath)
   {
@@ -2086,7 +2064,7 @@ vector<Issue> SetupService::FindIssues(bool checkPath, bool checkPackageIntegrit
           IssueType::RootDirectoryCoverage,
           IssueSeverity::Major,
           fmt::format(T_("Root directory #{0} is covered by root directory #{1}."), idx, idx2),
-          T_("") // TODO
+          "" // TODO
         });
       }
       else if (Utils::IsParentDirectoryOf(roots[idx].path, roots[idx2].path))
@@ -2095,7 +2073,7 @@ vector<Issue> SetupService::FindIssues(bool checkPath, bool checkPackageIntegrit
           IssueType::RootDirectoryCoverage,
           IssueSeverity::Major,
           fmt::format(T_("Root directory #{0} covers root directory #{1}."), idx, idx2),
-          T_("") // TODO
+          "" // TODO
         });
       }
     }
@@ -2116,7 +2094,7 @@ vector<Issue> SetupService::FindIssues(bool checkPath, bool checkPackageIntegrit
             IssueType::PackageDamaged,
             IssueSeverity::Critical,
             fmt::format(T_("Package {0} has been tampered with."), packageInfo.id),
-            T_("") // TODO
+            "" // TODO
           });
         }
       }
@@ -2133,10 +2111,9 @@ vector<Issue> SetupService::FindIssues(bool checkPath, bool checkPackageIntegrit
   return result;
 }
 
-vector<Issue> SetupService::GetIssues()
+vector<Issue> SetupServiceImpl::GetIssues()
 {
   vector<Setup::Issue> issues;
-  shared_ptr<Session> session = Session::Get();
   PathName issuesJson = session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_ISSUES_JSON);
   if (File::Exists(issuesJson))
   {
