@@ -1,6 +1,6 @@
 /* mikuiqt.cpp:
 
-   Copyright (C) 2008-2020 Christian Schenk
+   Copyright (C) 2008-2021 Christian Schenk
 
    This file is part of the MiKTeX UI Library.
 
@@ -23,16 +23,21 @@
 #include <mach-o/dyld.h>
 #endif
 
-#include <miktex/Core/ConfigNames>
+#include <miktex/Configuration/ConfigNames>
+#include <miktex/Locale/Translator>
 
 #include "internal.h"
 
+#include <QTranslator>
+
 #include "miktex/UI/UI"
 
 #include "InstallPackageDialog.h"
 #include "ProxyAuthenticationDialog.h"
 
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
+using namespace MiKTeX::Locale;
 using namespace MiKTeX::Packages;
 using namespace MiKTeX::UI::Qt;
 using namespace MiKTeX::Util;
@@ -67,6 +72,50 @@ PathName GetExecutableDir()
 }
 #endif
 
+static vector<unique_ptr<QTranslator>> qTranslators;
+
+static void InstallTranslators(shared_ptr<Session> session)
+{
+  auto uiLsystemUiLanguages = Translator("miktex-console", nullptr, session).GetSystemUILanguages();
+  if (uiLsystemUiLanguages.empty())
+  {
+    return;
+  }
+  reverse(uiLsystemUiLanguages.begin(), uiLsystemUiLanguages.end());
+  vector<string> domains{ "ui" };
+  vector<PathName> searchPath{ PathName(":/i18n") };
+  string localeDir;
+  if (session->TryGetConfigValue("Translator", "BaseDir", localeDir))
+  {
+    searchPath.push_back(PathName(localeDir));
+  }
+  for (const auto& uiLang : uiLsystemUiLanguages)
+  {
+    QLocale uiLocale = QLocale(QString::fromStdString(uiLang));
+    for (const auto& domain : domains)
+    {
+      for (const auto& path : searchPath)
+      {
+        auto qTranslator = make_unique<QTranslator>();
+        if (qTranslator->load(uiLocale, QString::fromStdString(domain), "_", QString::fromStdString(path.ToString()), ".qm"))
+        {
+          QCoreApplication::installTranslator(qTranslator.get());
+          qTranslators.push_back(move(qTranslator));
+        }
+      }
+    }
+  }
+}
+
+static void RemoveTranslators()
+{
+  for (auto& t : qTranslators)
+  {
+    QCoreApplication::removeTranslator(t.get());
+  }
+  qTranslators.clear();
+}
+
 MIKTEXUIQTEXPORT void MIKTEXCEECALL MiKTeX::UI::Qt::InitializeFramework()
 {
 #if defined(MIKTEX_WINDOWS)
@@ -90,12 +139,14 @@ MIKTEXUIQTEXPORT void MIKTEXCEECALL MiKTeX::UI::Qt::InitializeFramework()
   static int argc = 1;
   static PathName argv0;
   static char* argv[2];
-  argv0 = Session::Get()->GetMyProgramFile(false).GetFileNameWithoutExtension();
+  auto session = Session::Get();
+  argv0 = session->GetMyProgramFile(false).GetFileNameWithoutExtension();
   argv[0] = argv0.GetData();
   argv[1] = nullptr;
   if (useGUI)
   {
     application = new QApplication(argc, argv);
+    InstallTranslators(session);
   }
   else
   {
@@ -105,6 +156,7 @@ MIKTEXUIQTEXPORT void MIKTEXCEECALL MiKTeX::UI::Qt::InitializeFramework()
 
 MIKTEXUIQTEXPORT void MIKTEXCEECALL MiKTeX::UI::Qt::FinalizeFramework()
 {
+  RemoveTranslators();
   delete application;
   application = nullptr;
 }
