@@ -1,4 +1,4 @@
-/* $Id: dospecial.c 50765 2019-04-04 22:03:56Z karl $
+/* $Id$
  *   This routine handles special commands;
  *   predospecial() is for the prescan, dospecial() for the real thing.
  */
@@ -286,7 +286,11 @@ GetKeyVal(char *str, int *tno) /* returns NULL if none found, else next scan poi
      /* str : starting point for scan */
      /* tno : table entry number of keyword, or -1 if keyword not found */
 {
-   register char *s;
+#if defined(MIKTEX)
+  char* s;
+#else
+   unsigned char *s;
+#endif
    register int i;
    register char t;
 
@@ -400,7 +404,7 @@ predospecial(integer numbytes, Boolean scanning)
       p++;
 #ifdef DEBUG
    if (dd(D_SPECIAL))
-      fprintf(stderr, "Preprocessing special: %s\n", p);
+      fprintf_str(stderr, "Preprocessing special: %s\n", p);
 #endif
 
 /*
@@ -638,7 +642,7 @@ if (HPS_FLAG && NEED_NEW_BOX) {
       p++;
 #ifdef DEBUG
    if (dd(D_SPECIAL))
-      fprintf(stderr, "Processing special: %s\n", p);
+      fprintf_str(stderr, "Processing special: %s\n", p);
 #endif
 
    switch (*p) {
@@ -843,36 +847,37 @@ case 'h':
    if (strncmp(p, "header", 6)==0) return;
 #ifdef HPS
    if (strncmp(p, "html:", 5)==0) {
-     if (! HPS_FLAG) return;
-	 		p += 5;
-			while (isspace((unsigned char)*p))
-   		p++;
-			if (*p == '<') {
-    			char               *sp = p;
-    			char               *str;
-    			int                 ii=0;int len;int lower_len;
+      if (! HPS_FLAG) return;
+      p += 5;
+      while (isspace((unsigned char)*p))
+         p++;
+      if (*p == '<') {
+         char *sp = p;
+         char *str;
+         int  ii=0, len, lower_len;
 
-    			while ((*p) && (*p != '>')) {
-						ii++;
-						p++;
-   			 }
-    		str = (char *)mymalloc(ii+2);
-   			strncpy(str,sp+1,ii-1);
-    		str[ii-1] = 0;len=strlen(str);
-				if(len>6) lower_len=6; else lower_len=len;
-				for(ii=0;ii<lower_len;ii++) str[ii]=tolower((unsigned char)str[ii]);
-				do_html(str);
-   			free(str);
-				} else
+         while ((*p) && (*p != '>')) {
+            ii++;
+            p++;
+         }
+         str = (char *)mymalloc(ii+2);
+         strncpy(str,sp+1,ii-1);
+         str[ii-1] = 0;len=strlen(str);
+         if(len>6) lower_len=6; else lower_len=len;
+         for(ii=0;ii<lower_len;ii++) str[ii]=tolower((unsigned char)str[ii]);
+         do_html(str);
+         free(str);
+      } else
 #ifdef KPATHSEA
-				  if (!kpse_tex_hush ("special"))
+         if (!kpse_tex_hush ("special"))
 #endif
-				    {
+      {
 
-    			printf("Error in html special\n");
-    			return;
-				}
-	return;
+         sprintf(errbuf,"Error in html special\n");
+         error(errbuf);
+         return;
+      }
+      return;
    }
 #else
    if (strncmp(p, "html:", 5)==0) return;
