@@ -588,10 +588,8 @@ void Export()
       size_t count=0;
       do {
         trBeginTile(tr);
-        fpu_trap(false); // Work around FE_INVALID in OSMesa.
         remesh=true;
         drawscene(fullWidth,fullHeight);
-        fpu_trap(settings::trap());
         ++count;
       } while (trEndTile(tr));
       if(settings::verbose > 1)
@@ -1524,7 +1522,10 @@ void init()
   glutInitContextProfile(GLUT_CORE_PROFILE);
 #endif
 
+  fpu_trap(false); // Work around FE_INVALID
   glutInit(&argc,argv);
+  fpu_trap(settings::trap());
+
   screenWidth=glutGet(GLUT_SCREEN_WIDTH);
   screenHeight=glutGet(GLUT_SCREEN_HEIGHT);
 #endif
@@ -1625,8 +1626,10 @@ void glrender(const string& prefix, const picture *pic, const string& format,
 
 #ifdef HAVE_GL
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
+#ifndef HAVE_LIBOSMESA
   static bool initializedView=false;
 #endif
+#endif
 
 #ifdef HAVE_LIBOSMESA
   if(!webgl) {
@@ -1786,7 +1789,9 @@ void glrender(const string& prefix, const picture *pic, const string& format,
 #endif
 #endif
       string title=string(settings::PROGRAM)+": "+prefix;
+      fpu_trap(false); // Work around FE_INVALID
       window=glutCreateWindow(title.c_str());
+      fpu_trap(settings::trap());
 
       GLint samplebuf[1];
       glGetIntegerv(GL_SAMPLES,samplebuf);
@@ -1815,7 +1820,7 @@ void glrender(const string& prefix, const picture *pic, const string& format,
   } else if(!havewindow) {
     glutInitWindowSize(maxTileWidth,maxTileHeight);
     glutInitDisplayMode(displaymode);
-    fpu_trap(false); // Work around FE_INVALID in Gallium
+    fpu_trap(false); // Work around FE_INVALID
     window=glutCreateWindow("");
     fpu_trap(settings::trap());
     glutHideWindow();
@@ -1873,7 +1878,9 @@ void glrender(const string& prefix, const picture *pic, const string& format,
   if(View) {
 #ifdef HAVE_LIBGLUT
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
+#ifndef HAVE_LIBOSMESA
     initializedView=true;
+#endif
 #endif
     glutReshapeFunc(reshape);
     glutKeyboardFunc(keyboard);
@@ -2049,7 +2056,9 @@ void drawBuffer(vertexBuffer& data, GLint shader)
     glEnableVertexAttribArray(colorAttrib);
   }
 
+  fpu_trap(false); // Work around FE_INVALID
   glDrawElements(data.type,data.indices.size(),GL_UNSIGNED_INT,(void *) 0);
+  fpu_trap(settings::trap());
 
   glDisableVertexAttribArray(positionAttrib);
   if(normal && gl::Nlights > 0)
