@@ -63,23 +63,25 @@ bool warn(const string& s);
 extern string systemDir;
 extern string docdir;
 extern const string dirsep;
-  
+extern string tempdir;
+
 extern bool safe;
-  
+
+bool globalread();
 bool globalwrite();
 
 extern const string suffix;
 extern const string guisuffix;
 extern const string standardprefix;
-  
+
 extern string historyname;
-  
+
 void SetPageDimensions();
 
 types::record *getSettingsModule();
 
 vm::item& Setting(string name);
-  
+
 template <typename T>
 inline T getSetting(string name)
 {
@@ -92,7 +94,7 @@ extern bool gray;
 extern bool bw;
 extern bool rgb;
 extern bool cmyk;
-  
+
 bool view();
 bool trap();
 string outname();
@@ -102,23 +104,23 @@ void setOptions(int argc, char *argv[]);
 // Access the arguments once options have been parsed.
 int numArgs();
 char *getArg(int n);
- 
+
 Int getScroll();
-  
+
 #if defined(MIKTEX_WINDOWS)
 typedef int mode_t;
 #endif
 extern mode_t mask;
-  
+
 bool xe(const string& texengine);
 bool lua(const string& texengine);
 bool pdf(const string& texengine);
 bool latex(const string& texengine);
 bool context(const string& texengine);
-  
+
 string nativeformat();
 string defaultformat();
-  
+
 const char *beginlabel(const string& texengine);
 const char *endlabel(const string& texengine);
 const char *rawpostscript(const string& texengine);
@@ -126,10 +128,10 @@ const char *beginpicture(const string& texengine);
 const char *endpicture(const string& texengine);
 const char *beginspecial(const string& texengine);
 const char *endspecial();
-  
+
 string texcommand();
 string texprogram();
-  
+
 const double inches=72.0;
 const double cm=inches/2.54;
 const double tex2ps=72.0/72.27;
