@@ -41,9 +41,11 @@ authorization from the copyright holders.
 #endif
 #include <w2c/config.h>
 
-#ifndef POPPLER_VERSION
-#include <poppler-config.h>
-#endif
+/* 
+ * From TeX Live 2021, we use pplib by Pawe\l Jackowski instead of
+ * libpoppler
+ */
+#include <ppapi.h>
 
 #include <png.h>
 #include <zlib.h>
@@ -195,7 +197,7 @@ void initversionstring(char **versions)
         "Compiled with Graphite2 version %d.%d.%d; using %d.%d.%d\n"
         "Compiled with HarfBuzz version %s; using %s\n"
         "Compiled with libpng version %s; using %s\n"
-        "Compiled with poppler version %s\n"
+        "Compiled with pplib version %s\n"
 #ifdef XETEX_MAC
         "Using Mac OS X Core Text and Cocoa frameworks\n"
 #else
@@ -212,7 +214,7 @@ void initversionstring(char **versions)
             + strlen(hb_version_string())
             + strlen(PNG_LIBPNG_VER_STRING)
             + strlen(png_libpng_ver)
-            + strlen(POPPLER_VERSION)
+            + strlen(pplib_version)
 #ifndef XETEX_MAC
             + 6 * 3 /* for fontconfig version #s (won't really need 3 digits per field!) */
 #endif
@@ -242,7 +244,7 @@ void initversionstring(char **versions)
         GR2_VERSION_MAJOR, GR2_VERSION_MINOR, GR2_VERSION_BUGFIX,
         grMajor, grMinor, grBugfix,
         HB_VERSION_STRING, hb_version_string(),
-        PNG_LIBPNG_VER_STRING, png_libpng_ver, POPPLER_VERSION
+        PNG_LIBPNG_VER_STRING, png_libpng_ver, pplib_version
 #ifndef XETEX_MAC
         ,
         FC_VERSION / 10000, (FC_VERSION % 10000) / 100, FC_VERSION % 100,
@@ -2720,7 +2722,7 @@ boolean open_dvi_output(C4P::FileRoot& dviFile)
   std::shared_ptr<MiKTeX::Core::Session> session = MiKTeX::Core::Session::Get();
   if (nopdfoutput)
   {
-    MiKTeX::Core::PathName outPath;
+    MiKTeX::Util::PathName outPath;
     bool done = MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->OpenOutputFile(dviFile, MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->GetNameOfFile(), false, outPath);
     if (done)
     {
@@ -2730,7 +2732,7 @@ boolean open_dvi_output(C4P::FileRoot& dviFile)
   }
   else
   {
-    MiKTeX::Core::PathName dvipdfmx;
+    MiKTeX::Util::PathName dvipdfmx;
     if (!session->FindFile(dvipdfmxExecutable, MiKTeX::Core::FileType::EXE, dvipdfmx))
     {
       return 0;
@@ -2740,17 +2742,17 @@ boolean open_dvi_output(C4P::FileRoot& dviFile)
     processStartInfo.Arguments.push_back(dvipdfmxExecutable);
     switch (MiKTeX::App::Application::GetApplication()->GetEnableInstaller())
     {
-    case MiKTeX::Core::TriState::False:
+    case MiKTeX::Configuration::TriState::False:
       processStartInfo.Arguments.push_back("--miktex-disable-installer");
       break;
-    case MiKTeX::Core::TriState::True:
+    case MiKTeX::Configuration::TriState::True:
       processStartInfo.Arguments.push_back("--miktex-enable-installer");
       break;
     default:
       break;
     }
     processStartInfo.Arguments.insert(processStartInfo.Arguments.end(), dvipdfmxArgs.begin(), dvipdfmxArgs.end());
-    MiKTeX::Core::PathName outPath = MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->GetOutputDirectory() / MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->GetNameOfFile();
+    MiKTeX::Util::PathName outPath = MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->GetOutputDirectory() / MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->GetNameOfFile();
     processStartInfo.Arguments.push_back("-o");
     processStartInfo.Arguments.push_back(outPath.ToString());
     if (papersize != nullptr)
